@@ -70,6 +70,10 @@ class dbd_bot():
 		self._codes={}
 		self._cooldown={}
 		self._db=_variables()
+		try:
+			self._debug=info["debug"]=="true"
+		except:
+			self._debug=False
 		try:
 			self._separator=info["sep"]
 		except:
@@ -91,7 +95,10 @@ class dbd_bot():
 			self._style=info["style"]
 		except:
 			self._style="[]"
-		self._client=commands.Bot(command_prefix=self._prefix, intents=discord.Intents.all())
+		if self._self:
+			self._client=commands.Bot(command_prefix=self._prefix, intents=discord.Intents.all())
+		else:
+			self._client=commands.Bot(command_prefix=self._prefix, intents=discord.Intents.all())
 		self._client.remove_command(help)
 
 	def command(self, info):
@@ -221,7 +228,7 @@ def _start(client, bot):
 					except:
 						break
 		return line
-	async def replaces(line, message, guild, content, msg, vars, timerep):
+	async def replaces(line, message, guild, content, msg, vars, timerep, author):
 		nowloop=""
 		content2=content
 		try:
@@ -235,13 +242,31 @@ def _start(client, bot):
 		line=line.replace("%time%",str(timerep))
 		line=line.replace("$uptime",str(int(time.time() - startTime)))
 		line=line.replace("$ping",str(round(client.latency*800)))
+		cs=0
+		for c in bot._codes:
+			if c.startswith("command:"):
+				cs+=1
+		line=line.replace("$commandsCount",str(cs))
 		try:
+			line=line.replace("$membersCount",str(len(guild.members)-1))
+			check=[]
+			mems=0-len(client.guilds)+1
+			for g in client.guilds:
+				mms=len(g.members)
+				for m in g.members:
+					if not m in check:
+						check.append(m)
+						mems+=1
+			line=line.replace("$allMembersCount",str(mems))
 			line=line.replace("$serverID",str(guild.id))
 		except:
 			pass
+		try:
+			line=line.replace("$authorID",str(author.id))
+		except:
+			pass
 		try:
 			line=line.replace("$channelID",str(message.channel.id))
-			line=line.replace("$authorID",str(message.author.id))
 			line=line.replace("$triggerID",str(message.id))
 			line=line.replace("$messageID",str(msg.id))
 		except:
@@ -276,11 +301,26 @@ def _start(client, bot):
 				except Exception as e:
 					try:
 						if line.split("$mentioned"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[1]=="yes":
-							line=line.replace("$mentioned"+bot._style[0]+line.split("$mentioned"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], str(message.author.id))
+							line=line.replace("$mentioned"+bot._style[0]+line.split("$mentioned"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], str(author.id))
 						else:
 							line=line.replace("$mentioned"+bot._style[0]+line.split("$mentioned"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], "none")
 					except:
 						line=line.replace("$mentioned"+bot._style[0]+line.split("$mentioned"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], "none")
+		while "$mentionedRole"+bot._style[0] in line:
+			if nowloop==line:
+				break
+			else:
+				nowloop=line
+				try:
+					line=line.replace("$mentionedRole"+bot._style[0]+line.split("$mentionedRole"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], str(message.role_mentions[int(line.split("$mentionedRole"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0])-1].id))
+				except Exception as e:
+					try:
+						if line.split("$mentionedRole"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[1]=="yes":
+							line=line.replace("$mentionedRole"+bot._style[0]+line.split("$mentionedRole"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], str(author.id))
+						else:
+							line=line.replace("$mentionedRole"+bot._style[0]+line.split("$mentionedRole"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], "none")
+					except:
+						line=line.replace("$mentionedRole"+bot._style[0]+line.split("$mentionedRole"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1], "none")
 		while "$roleID"+bot._style[0] in line:
 			if nowloop==line:
 				break
@@ -338,14 +378,14 @@ def _start(client, bot):
 				nowloop=line
 				try:
 					userstr=""
-					mes=await message.channel.fetch_message(int(line.split("$getReactions"+bot._style[0])[-1].split(bot._style[1])[0].split(";")[0]))
+					mes=await message.channel.fetch_message(int(line.split("$getReactions"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]))
 					mes=mes.reactions
 					for react in mes:
-						if react.emoji==line.split("$getReactions"+bot._style[0])[-1].split(bot._style[1])[0].split(";")[1]:
+						if react.emoji==line.split("$getReactions"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[1]:
 							users = await react.users().flatten()
 							for user in users:
 								if user.id!=client.user.id:
-									userstr+=str(user.id)+";"
+									userstr+=str(user.id)+","
 					userstr=userstr[:-1]
 					line=line.replace("$getReactions"+bot._style[0]+line.split("$getReactions"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],userstr)
 				except Exception as e:
@@ -366,7 +406,10 @@ def _start(client, bot):
 			else:
 				nowloop=line
 				try:
-					r=random.randint(int(line.split("$random"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]),int(line.split("$random"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[1]))
+					r=[int(line.split("$random"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[0]),int(line.split("$random"+bot._style[0])[-1].split(bot._style[1])[0].split(bot._separator)[1])]
+					if r[0]>r[1]:
+						r=[r[1],r[0]]
+					r=random.randint(int(r[0]),int(r[1]))
 					line=line.replace("$random"+bot._style[0]+line.split("$random"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],str(r))
 				except:
 					line=line.replace("$random"+bot._style[0]+line.split("$random"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
@@ -417,6 +460,16 @@ def _start(client, bot):
 					line=line.replace("$avatar"+bot._style[0]+line.split("$avatar"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],str(user.avatar_url))
 				except Exception as e:
 					line=line.replace("$avatar"+bot._style[0]+line.split("$avatar"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
+		while "$emojiUrl"+bot._style[0] in line:
+			if nowloop==line:
+				break
+			else:
+				nowloop=line
+				try:
+					emoji = discord.utils.get(guild.emojis, id=int(line.split("$emojiUrl"+bot._style[0])[-1].split(bot._style[1])[0]))
+					line=line.replace("$emojiUrl"+bot._style[0]+line.split("$emojiUrl"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],str(emoji.url))
+				except Exception as e:
+					line=line.replace("$emojiUrl"+bot._style[0]+line.split("$emojiUrl"+bot._style[0])[-1].split(bot._style[1])[0]+bot._style[1],"none")
 		line=get_vars(line, guild, vars)
 		while "$replaceText"+bot._style[0] in line:
 			if nowloop==line:
@@ -511,11 +564,19 @@ def _start(client, bot):
 			restart=False
 			go=0
 			lines=[]
-			msg=""
-			if name=="bot_ready":
-				content="$message"
+			msg=""		
+			ctx=""
+			author=""
+			content="$message"
+			if name=="botReady":
+				guild=""
+			elif name=="onJoined":
+				guild=message.guild
+				author=message
 			else:
+				guild=message.guild
 				ctx=message.channel
+				author=message.author
 				content=message.content.lstrip(trigger).lstrip(" ")
 			for line in bot._codes[name]["code"]:
 				if "$eval" in line:
@@ -526,15 +587,8 @@ def _start(client, bot):
 			if "" in lines:
 				lines.remove("")
 			for line in lines:
+				beforeline=line
 				if go==0:
-					guild=message
-					try:
-						message.roles
-					except:
-						try:
-							guild=message.guild
-						except:
-							guild=""
 					line=line.lstrip(" ")
 					try:
 						if line.startswith("$cooldown"+bot._style[0]):
@@ -543,39 +597,38 @@ def _start(client, bot):
 								line=line[:-1]
 								if not name in bot._cooldown:
 									bot._cooldown[name]={"user":{},"server":{}, "global":0}
-								if not str(message.author.id) in bot._cooldown[name]["user"]:
-									bot._cooldown[name]["user"][str(message.author.id)]=0
+								if not str(author.id) in bot._cooldown[name]["user"]:
+									bot._cooldown[name]["user"][str(author.id)]=0
 								if not str(guild.id) in bot._cooldown[name]["server"]:
 									bot._cooldown[name]["server"][str(guild.id)]=0
 								try:
-									if bot._cooldown[name]["user"][str(message.author.id)]-int(time.time() - startTime)<0:
+									if bot._cooldown[name]["user"][str(author.id)]-int(time.time() - startTime)<0:
 										timerep="0"
 									else:
-										timerep=str(bot._cooldown[name]["user"][str(message.author.id)]-int(time.time() - startTime))
+										timerep=str(bot._cooldown[name]["user"][str(author.id)]-int(time.time() - startTime))
 								except:
 									timerep="0"
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
-								cd=line.split(bot._separator)[1]
-								if bot._cooldown[name]["user"][str(message.author.id)]>int(time.time() - startTime):
-									go+=int(line.split(bot._separator)[0])
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
+								if bot._cooldown[name]["user"][str(author.id)]>int(time.time() - startTime):
+									pass
 								else:
-									if cd.endswith("s"):
-										cd=cd.rstrip("s")
-									elif cd.endswith("m"):
-										cd=cd.rstrip("m")
-										cd=int(cd)*60
-									elif cd.endswith("h"):
-										cd=cd.rstrip("h")
-										cd=int(cd)*60*60
-									elif cd.endswith("d"):
-										cd=cd.rstrip("d")
-										cd=int(cd)*60*60*24
-									bot._cooldown[name]["user"][str(message.author.id)]=int(time.time() - startTime)+int(cd)
+									if line.endswith("s"):
+										line=line.rstrip("s")
+									elif line.endswith("m"):
+										line=line.rstrip("m")
+										line=int(line)*60
+									elif line.endswith("h"):
+										line=line.rstrip("h")
+										line=int(line)*60*60
+									elif line.endswith("d"):
+										line=line.rstrip("d")
+										line=int(line)*60*60*24
+									bot._cooldown[name]["user"][str(author.id)]=int(time.time() - startTime)+int(line)
 						elif line.startswith("$deletemessage"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$deletemessage"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								mes=await ctx.fetch_message(int(line))
 								await mes.delete()
 						elif line.startswith("$serverCooldown"+bot._style[0]):
@@ -584,8 +637,8 @@ def _start(client, bot):
 								line=line[:-1]
 								if not name in bot._cooldown:
 									bot._cooldown[name]={"user":{},"server":{}, "global":0}
-								if not str(message.author.id) in bot._cooldown[name]["user"]:
-									bot._cooldown[name]["user"][str(message.author.id)]=0
+								if not str(author.id) in bot._cooldown[name]["user"]:
+									bot._cooldown[name]["user"][str(author.id)]=0
 								if not str(guild.id) in bot._cooldown[name]["server"]:
 									bot._cooldown[name]["server"][str(guild.id)]=0
 								try:
@@ -595,31 +648,30 @@ def _start(client, bot):
 										timerep=str(bot._cooldown[name]["server"][str(guild.id)]-int(time.time() - startTime))
 								except:
 									timerep="0"
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
-								cd=line.split(bot._separator)[1]
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								if bot._cooldown[name]["server"][str(guild.id)]>int(time.time() - startTime):
-									go+=int(line.split(bot._separator)[0])
+									pass
 								else:
-									if cd.endswith("s"):
-										cd=cd.rstrip("s")
-									elif cd.endswith("m"):
-										cd=cd.rstrip("m")
-										cd=int(cd)*60
-									elif cd.endswith("h"):
-										cd=cd.rstrip("h")
-										cd=int(cd)*60*60
-									elif cd.endswith("d"):
-										cd=cd.rstrip("d")
-										cd=int(cd)*60*60*24
-									bot._cooldown[name]["server"][str(guild.id)]=int(time.time() - startTime)+int(cd)
+									if line.endswith("s"):
+										line=line.rstrip("s")
+									elif line.endswith("m"):
+										line=line.rstrip("m")
+										line=int(line)*60
+									elif line.endswith("h"):
+										line=line.rstrip("h")
+										line=int(line)*60*60
+									elif line.endswith("d"):
+										line=line.rstrip("d")
+										line=int(line)*60*60*24
+									bot._cooldown[name]["server"][str(guild.id)]=int(time.time() - startTime)+int(line)
 						elif line.startswith("$globalCooldown"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$globalCooldown"+bot._style[0]):]
 								line=line[:-1]
 								if not name in bot._cooldown:
 									bot._cooldown[name]={"user":{},"server":{}, "global":0}
-								if not str(message.author.id) in bot._cooldown[name]["user"]:
-									bot._cooldown[name]["user"][str(message.author.id)]=0
+								if not str(author.id) in bot._cooldown[name]["user"]:
+									bot._cooldown[name]["user"][str(author.id)]=0
 								if not str(guild.id) in bot._cooldown[name]["server"]:
 									bot._cooldown[name]["server"][str(guild.id)]=0
 								try:
@@ -629,35 +681,34 @@ def _start(client, bot):
 										timerep=str(bot._cooldown[name]["global"]-int(time.time() - startTime))
 								except:
 									timerep="0"
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
-								cd=line.split(bot._separator)[0]
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								if bot._cooldown[name]["global"]>int(time.time() - startTime):
-									go+=int(line.split(bot._separator)[0])
+									pass
 								else:
-									if cd.endswith("s"):
-										cd=cd.rstrip("s")
-									elif cd.endswith("m"):
-										cd=cd.rstrip("m")
-										cd=int(cd)*60
-									elif cd.endswith("h"):
-										cd=cd.rstrip("h")
-										cd=int(cd)*60*60
-									elif cd.endswith("d"):
-										cd=cd.rstrip("d")
-										cd=int(cd)*60*60*24
-									bot._cooldown[name]["global"]=int(time.time() - startTime)+int(cd)
+									if line.endswith("s"):
+										line=line.rstrip("s")
+									elif line.endswith("m"):
+										line=line.rstrip("m")
+										line=int(line)*60
+									elif line.endswith("h"):
+										line=line.rstrip("h")
+										line=int(line)*60*60
+									elif line.endswith("d"):
+										line=line.rstrip("d")
+										line=int(line)*60*60*24
+									bot._cooldown[name]["global"]=int(time.time() - startTime)+int(line)
 						elif line.startswith("$reply"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$reply"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								mes=await ctx.fetch_message(int(line.split(bot._separator)[0]))
 								msg=await mes.reply(content=line.split(bot._separator)[1])
 						elif line.startswith("$embedReply"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$embedReply"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								mes=await ctx.fetch_message(int(line.split("&")[0]))
 								emb=embed(line.split("&")[1])
 								msg=await mes.reply(embed=emb)
@@ -665,15 +716,15 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$useChannel"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								if discord.utils.get(guild.channels, id=int(line)):
 									ctx=discord.utils.get(guild.channels, id=int(line))
 						elif line.startswith("$reactionPages"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$reactionPages"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
-								userid = str(message.author.id)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
+								userid = str(author.id)
 								previuspage = line.split("&")[0].split(bot._separator)[0]
 								nextpage = line.split("&")[0].split(bot._separator)[1]
 								page = 0
@@ -701,7 +752,7 @@ def _start(client, bot):
 												page=page
 											r=previuspage
 										nowpage=descript[page]
-										await mesg.remove_reaction(r,message.author)
+										await mesg.remove_reaction(r,author)
 										await mesg.edit(content=nowpage)
 									except asyncio.TimeoutError:
 										try:
@@ -717,8 +768,8 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$embedReactionPages"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
-								userid = str(message.author.id)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
+								userid = str(author.id)
 								previuspage = line.split("&")[0].split(bot._separator)[0]
 								nextpage = line.split("&")[0].split(bot._separator)[1]
 								page = 0
@@ -746,7 +797,7 @@ def _start(client, bot):
 												page=page
 											r=previuspage
 										nowpage=embed(descript[page])
-										await mesg.remove_reaction(r,message.author)
+										await mesg.remove_reaction(r,author)
 										await mesg.edit(embed=nowpage)
 									except asyncio.TimeoutError:
 										try:
@@ -762,9 +813,16 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$useServer"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								if discord.utils.get(client.guilds, id=int(line)):
-									ctx=discord.utils.get(client.guilds, id=int(line))
+									guild=discord.utils.get(client.guilds, id=int(line))
+						elif line.startswith("$useUser"+bot._style[0]):
+							if line[-1]==bot._style[1]:
+								line=line[len("$useUser"+bot._style[0]):]
+								line=line[:-1]
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
+								if discord.utils.get(guild.members, id=int(line)):
+									ctx=discord.utils.get(guild.members, id=int(line))
 						elif line.startswith("$stop"):
 							stop=True
 							break
@@ -772,11 +830,11 @@ def _start(client, bot):
 							line=line[len("$onlyIDs"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=line.split(bot._separator)
 								users=line
 								users.remove(line[-1])
-								if not str(message.author.id) in users:
+								if not str(author.id) in users:
 									if line!="":
 										await ctx.send(line)
 									break
@@ -784,7 +842,7 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$giveRole"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=line.split(bot._separator)
 								role = discord.utils.get(guild.roles, id=int(line[1]))
 								user=discord.utils.get(guild.members, id=int(line[0]))
@@ -793,7 +851,7 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$takeRole"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=line.split(bot._separator)
 								role = discord.utils.get(guild.roles, id=int(line[1]))
 								user=discord.utils.get(guild.members, id=int(line[0]))
@@ -802,21 +860,21 @@ def _start(client, bot):
 							line=line[len("$kick"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								user=discord.utils.get(message.guild.members, id=int(line))
 								await user.kick(reason="")
 						elif line.startswith("$ban"+bot._style[0]):
 							line=line[len("$ban"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								user=discord.utils.get(message.guild.members, id=int(line))
 								await user.ban(reason="")
 						elif line.startswith("$editEmbed"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$editEmbed"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								emb=embed(line)
 								try:
 									await msg.edit(embed=emb)
@@ -826,7 +884,7 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$sendEmbed"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								emb=embed(line)
 								try:
 									msg=await ctx.send(embed=emb)
@@ -836,7 +894,7 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$unban"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								banned_users = await message.bans()
 								for ban_entry in banned_users:
 									user = ban_entry.user
@@ -846,28 +904,28 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$setServerVar"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=line.split(bot._separator)
 								bot._db.set("server",line[0],line[1],line[2])
 						elif line.startswith("$setVar"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$setVar"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=line.split(bot._separator)
 								bot._db.set("global",line[0],line[1],line[2])
 						elif line.startswith("$setLocalVar"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$setLocalVar"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=line.split(bot._separator)
 								vars[line[0]]=line[1]
 						elif line.startswith("$setStatus"+bot._style[0]):
 							if line[-1]==bot._style[1]:
 								line=line[len("$setStatus"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=[line.split(bot._separator)[0],line.split(bot._separator)[1], line[len(line.split(bot._separator)[0]+bot._separator+line.split(bot._separator)[1]+bot._separator):]]
 								status=discord.Status.online
 								if line[2]=="dnd":
@@ -890,7 +948,7 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$setUserVar"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								line=line.split(bot._separator)
 								try:
 									bot._db.set("user",line[0],line[1],line[2],guild.id)
@@ -900,7 +958,7 @@ def _start(client, bot):
 							if line[-1]==bot._style[1]:
 								line=line[len("$addReactions"+bot._style[0]):]
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								for react in line.split(bot._separator):
 									try:
 										await msg.add_reaction(react)
@@ -914,25 +972,25 @@ def _start(client, bot):
 							line=line[len("$print"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								print(line)
 						elif line.startswith("$send"+bot._style[0]):
 							line=line[len("$send"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								msg=await ctx.send(line)
 						elif line.startswith("$edit"+bot._style[0]):
 							line=line[len("$edit"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								await msg.edit(content=line)
 						elif line.startswith("$wait"+bot._style[0]):
 							line=line[len("$wait"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								if line.endswith("s"):
 									line=line.rstrip("s")
 								elif line.endswith("m"):
@@ -959,56 +1017,56 @@ def _start(client, bot):
 									line=line.lstrip(" ")
 								if "==" in line:
 									t=line.split("==")
-									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep)
-									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep)
+									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep, author)
+									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep, author)
 									if t[0]==t[1]:
 										t=True
 									else:
 										t=False
 								elif "!=" in line:
 									t=line.split("!=")
-									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep)
-									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep)
+									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep, author)
+									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep, author)
 									if t[0]!=t[1]:
 										t=True
 									else:
 										t=False
 								elif ">=" in line:
 									t=line.split(">=")
-									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep)
-									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep)
+									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep, author)
+									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep, author)
 									if int(t[0])>=int(t[1]):
 										t=True
 									else:
 										t=False
 								elif "<=" in line:
 									t=line.split("<=")
-									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep)
-									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep)
+									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep, author)
+									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep, author)
 									if int(t[0])<=int(t[1]):
 										t=True
 									else:
 										t=False
 								elif ">" in line:
 									t=line.split(">")
-									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep)
-									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep)
+									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep, author)
+									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep, author)
 									if int(t[0])>int(t[1]):
 										t=True
 									else:
 										t=False
 								elif "<" in line:
 									t=line.split("<")
-									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep)
-									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep)
+									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep, author)
+									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep, author)
 									if int(t[0])<int(t[1]):
 										t=True
 									else:
 										t=False
 								elif " in " in line:
 									t=line.split(" in ")
-									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep)
-									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep)
+									t[0]=await replaces(t[0], message, guild, content, msg, vars, timerep, author)
+									t[1]=await replaces(t[1], message, guild, content, msg, vars, timerep, author)
 									if t[0] in t[1]:
 										t=True
 									else:
@@ -1021,16 +1079,106 @@ def _start(client, bot):
 							line=line[len("$clear"+bot._style[0]):]
 							if line[-1]==bot._style[1]:
 								line=line[:-1]
-								line=await replaces(line, message, guild, content, msg, vars, timerep)
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
 								await message.channel.purge(limit=int(line)+1)
+						elif line.startswith("$screen"+bot._style[0]):
+							line=line[len("$screen"+bot._style[0]):]
+							if line[-1]==bot._style[1]:
+								from PIL import Image, ImageDraw, ImageFont, ImageFilter
+								from datetime import datetime
+								line=line[:-1]
+								line=await replaces(line, message, guild, content, msg, vars, timerep, author)
+								member=discord.utils.get(guild.members, id=int(line.split(bot._separator)[0]))
+								font = line.split(bot._separator)[1]
+								line=line[len(line.split(bot._separator)[0]+bot._separator+line.split(bot._separator)[1]+bot._separator):]
+								ns=0
+								for i in range(int((len(line))/49)):
+									i=i+1
+									line = line[:i*49] + "\n" + line[i*49:]
+								colour = {
+									"time": (114, 118, 125),
+									"content": (220, 221, 222)
+								}
+							
+								size = {
+									"title": 20,
+									"time": 13
+								}
+								ns=int((len(line))/49)
+								ns=ns-5
+								if ns<0:
+									ns=0
+								img = Image.new('RGB', (600, int(23*(5+ns))), color = (54,57,63))
+								titlefnt = ImageFont.truetype(font, size["title"])
+								timefnt = ImageFont.truetype(font, size["time"])
+								d = ImageDraw.Draw(img)
+								if member.nick is None:
+									txt = member.name
+								else:
+									txt = member.nick
+								color = member.color.to_rgb()
+								if color == (0, 0, 0):
+									color = (255,255,255)
+								d.text((90, 20), txt, font=titlefnt, fill=color)
+								h, w = d.textsize(txt, font=titlefnt)
+								curtime = datetime.now()
+								time = f"Сегодня, {curtime.hour}:{curtime.minute}"
+								d.text((90+h+10, 25), time, font=timefnt, fill=colour["time"])
+								d.text((90, 25+w), line, font=titlefnt, fill=colour["content"])
+								img.save('img.png')
+								if member.is_avatar_animated():
+									await member.avatar_url_as().save("pfp.gif")
+									f2 = Image.open("pfp.gif")
+								else:
+									await member.avatar_url_as(format="png").save("pfp.png")
+									f2 = Image.open("pfp.png")
+								f1 = Image.open("img.png")
+								f2.thumbnail((50, 55))
+								f2.save("pfp.png")
+							
+								f2 = Image.open("pfp.png").convert("RGB")
+							
+								mask = Image.new("L", f2.size, 0)
+								draw = ImageDraw.Draw(mask)
+								draw.ellipse((0, 0, f2.size[0], f2.size[1]), fill=255)
+								mask = mask.filter(ImageFilter.GaussianBlur(0))
+							
+								result = f2.copy()
+								result.putalpha(mask)
+							
+								result.save('pfp.png')
+							
+								f2 = Image.open("pfp.png")
+							
+								f3 = f1.copy()
+								f3.paste(f2, (20, 20), f2)
+								f3.save("img.png")
+							
+								file = discord.File("img.png")
+								await ctx.send(file=file)
+							
+								try:
+									os.remove("pfp.gif")
+								except:
+									pass
+								try:
+									os.remove("pfp.png")
+								except:
+									pass
+								try:
+									os.remove("img.png")
+								except:
+									pass
 						elif line.startswith("$replay"):
 							restart=True
 							break
 						else:
-							print(f"Неизвестная функция {line}")
+							if bot._debug:
+								print(f"Неизвестная функция {beforeline}")
 					except Exception as e:
-						print(f"Ошибка в функции {line}")
-						raise e
+						if bot._debug:
+							raise e
+							print(f"Ошибка в функции {beforeline}")
 				else:
 					go-=1
 			if stop:
@@ -1064,13 +1212,20 @@ def _start(client, bot):
 							bot._db.data["global"][str(m.id)][v]=bot._db.vars[v]
 		bot._db.save()
 		try:
-			await use("", "bot_ready", "")
+			await use("", "botReady", "")
 		except Exception as e:
 			pass
 
+	@client.event
+	async def on_member_join(user):
+		try:
+			await use(user, "onJoined", "")
+		except Exception as e:
+			pass
+	
 	@client.event
 	async def on_message(message):
-		if message.author.id!=client.user.id and not message.author.bot:
+		if not message.author.bot:
 			if str(message.content).startswith(bot._client.user.mention):
 				if bot._help:
 					cmds=""
@@ -1086,11 +1241,11 @@ def _start(client, bot):
 							triggers=triggers.rstrip("]")
 							triggers=triggers.split(bot._separator)
 							for trigger in triggers:
-								trigger=await replaces(str(bot._codes[cmd]["prefix"]+trigger), message, message.guild, message.content.lstrip(str(bot._codes[cmd]["prefix"]+trigger).lower()).lstrip(" "), "", {}, 0)
+								trigger=await replaces(str(bot._codes[cmd]["prefix"]+trigger), message, message.guild, message.content.lstrip(str(bot._codes[cmd]["prefix"]+trigger).lower()).lstrip(" "), "", {}, 0, "")
 								if str(message.content).lower().startswith(str(trigger).lower()):
 									await use(message, cmd, bot._codes[cmd]["prefix"]+trigger)
 						else:
-							trigger=await replaces(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"]), message, message.guild, message.content.lstrip(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"])).lstrip(" "), "", {}, 0)
+							trigger=await replaces(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"]), message, message.guild, message.content.lstrip(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"])).lstrip(" "), "", {}, 0, "")
 							if str(message.content).lower().startswith(trigger.lower()):
 								await use(message, cmd, bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"])
 					else:
@@ -1099,11 +1254,11 @@ def _start(client, bot):
 							triggers=triggers.rstrip("]")
 							triggers=triggers.split(bot._separator)
 							for trigger in triggers:
-								trigger=await replaces(str(bot._codes[cmd]["prefix"]+trigger), message, message.guild, message.content.lstrip(trigger).lstrip(" "), "", {}, 0)
+								trigger=await replaces(str(bot._codes[cmd]["prefix"]+trigger), message, message.guild, message.content.lstrip(trigger).lstrip(" "), "", {}, 0, "")
 								if str(message.content).startswith(trigger):
 									await use(message, cmd, bot._codes[cmd]["prefix"]+trigger)
 						else:
-							trigger=await replaces(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"]), message, message.guild, message.content.lstrip(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"])).lstrip(" "), "", {}, 0)
+							trigger=await replaces(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"]), message, message.guild, message.content.lstrip(str(bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"])).lstrip(" "), "", {}, 0, "")
 							if str(message.content).startswith(trigger):
 								await use(message, cmd, bot._codes[cmd]["prefix"]+bot._codes[cmd]["name"])
 	if bot._self:
